@@ -59,64 +59,42 @@
 #include "array_fun.h"
 #include "PieceStatMan.h"
 #include "wallclock.h"
+#include "bitfield.h"
+#ifdef ENABLE_BITTORRENT
+# include "bittorrent_helper.h"
+#endif // ENABLE_BITTORRENT
 
 namespace aria2 {
 
 DefaultPieceStorage::DefaultPieceStorage
 (const SharedHandle<DownloadContext>& downloadContext, const Option* option):
-  downloadContext(downloadContext),
-  bitfieldMan(new BitfieldMan(downloadContext->getPieceLength(),
-                              downloadContext->getTotalLength())),
-  _diskWriterFactory(new DefaultDiskWriterFactory()),
-  endGamePieceNum(END_GAME_PIECE_NUM),
-  logger(LogFactory::getInstance()),
-  option(option),
-  _pieceStatMan(new PieceStatMan(downloadContext->getNumPieces(), true)),
-  _pieceSelector(new RarestPieceSelector(_pieceStatMan))
+  downloadContext_(downloadContext),
+  bitfieldMan_(new BitfieldMan(downloadContext->getPieceLength(),
+                               downloadContext->getTotalLength())),
+  diskWriterFactory_(new DefaultDiskWriterFactory()),
+  endGame_(false),
+  endGamePieceNum_(END_GAME_PIECE_NUM),
+  logger_(LogFactory::getInstance()),
+  option_(option),
+  pieceStatMan_(new PieceStatMan(downloadContext->getNumPieces(), true)),
+  pieceSelector_(new RarestPieceSelector(pieceStatMan_))
 {}
 
 DefaultPieceStorage::~DefaultPieceStorage() {
-  delete bitfieldMan;
-}
-
-bool DefaultPieceStorage::isEndGame()
-{
-  return bitfieldMan->countMissingBlock() <= endGamePieceNum;
-}
-
-bool DefaultPieceStorage::getMissingPieceIndex(size_t& index,
-                                               const unsigned char* bitfield,
-                                               size_t length)
-{
-  const size_t mislen = bitfieldMan->getBitfieldLength();
-  array_ptr<unsigned char> misbitfield(new unsigned char[mislen]);
-  bool r;
-  if(isEndGame()) {
-    r = bitfieldMan->getAllMissingIndexes(misbitfield, mislen,
-                                          bitfield, length);
-  } else {
-    r = bitfieldMan->getAllMissingUnusedIndexes(misbitfield, mislen,
-                                                bitfield, length);
-  }
-  if(r) {
-    // We assume indexes is sorted using comparator less.
-    return _pieceSelector->select(index, misbitfield,bitfieldMan->countBlock());
-  } else {
-    return false;
-  }
+  delete bitfieldMan_;
 }
 
 SharedHandle<Piece> DefaultPieceStorage::checkOutPiece(size_t index)
 {
-  bitfieldMan->setUseBit(index);
+  bitfieldMan_->setUseBit(index);
 
   SharedHandle<Piece> piece = findUsedPiece(index);
   if(piece.isNull()) {
-    piece.reset(new Piece(index, bitfieldMan->getBlockLength(index)));
+    piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-    piece->setHashAlgo(downloadContext->getPieceHashAlgo());
+    piece->setHashAlgo(downloadContext_->getPieceHashAlgo());
 
 #endif // ENABLE_MESSAGE_DIGEST
 
@@ -134,10 +112,10 @@ SharedHandle<Piece> DefaultPieceStorage::checkOutPiece(size_t index)
 SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
 {
   SharedHandle<Piece> piece;
-  if(0 <= index && index <= bitfieldMan->getMaxIndex()) {
+  if(0 <= index && index <= bitfieldMan_->getMaxIndex()) {
     piece = findUsedPiece(index);
     if(piece.isNull()) {
-      piece.reset(new Piece(index, bitfieldMan->getBlockLength(index)));
+      piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
       if(hasPiece(index)) {
         piece->setAllBlock();
       }
@@ -149,11 +127,11 @@ SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
 void DefaultPieceStorage::addUsedPiece(const SharedHandle<Piece>& piece)
 {
   std::deque<SharedHandle<Piece> >::iterator i =
-    std::lower_bound(usedPieces.begin(), usedPieces.end(), piece);
-  usedPieces.insert(i, piece);
-  if(logger->debug()) {
-    logger->debug("usedPieces.size()=%lu",
-                  static_cast<unsigned long>(usedPieces.size()));
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece);
+  usedPieces_.insert(i, piece);
+  if(logger_->debug()) {
+    logger_->debug("usedPieces_.size()=%lu",
+                   static_cast<unsigned long>(usedPieces_.size()));
   }
 }
 
@@ -163,8 +141,8 @@ SharedHandle<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
   p->setIndex(index);
 
   std::deque<SharedHandle<Piece> >::const_iterator i =
-    std::lower_bound(usedPieces.begin(), usedPieces.end(), p);
-  if(i != usedPieces.end() && (*i) == p) {
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), p);
+  if(i != usedPieces_.end() && (*i) == p) {
     return *i;
   } else {
     p.reset(0);
@@ -172,59 +150,58 @@ SharedHandle<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
-(const unsigned char* bitfield, size_t length)
-{
-  size_t index;
-  if(getMissingPieceIndex(index, bitfield, length)) {
-    return checkOutPiece(index);
-  } else {
-    return SharedHandle<Piece>();
-  }
-}
-
-SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
-(const BitfieldMan& bitfield)
-{
-  return getMissingPiece(bitfield.getBitfield(), bitfield.getBitfieldLength());
-}
-
 #ifdef ENABLE_BITTORRENT
 
 bool DefaultPieceStorage::hasMissingPiece(const SharedHandle<Peer>& peer)
 {
-  return bitfieldMan->hasMissingPiece(peer->getBitfield(),
-                                      peer->getBitfieldLength());
+  return bitfieldMan_->hasMissingPiece(peer->getBitfield(),
+                                       peer->getBitfieldLength());
 }
 
-SharedHandle<Piece>
-DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
+void DefaultPieceStorage::getMissingPiece
+(std::vector<SharedHandle<Piece> >& pieces,
+ size_t minMissingBlocks,
+ const unsigned char* bitfield,
+ size_t length)
 {
-  return getMissingPiece(peer->getBitfield(), peer->getBitfieldLength());
-}
-
-void DefaultPieceStorage::createFastIndexBitfield
-(BitfieldMan& bitfield, const SharedHandle<Peer>& peer)
-{
-  for(std::vector<size_t>::const_iterator itr =
-        peer->getPeerAllowedIndexSet().begin(),
-        eoi = peer->getPeerAllowedIndexSet().end(); itr != eoi; ++itr) {
-    if(!bitfieldMan->isBitSet(*itr) && peer->hasPiece(*itr)) {
-      bitfield.setBit(*itr);
+  const size_t mislen = bitfieldMan_->getBitfieldLength();
+  array_ptr<unsigned char> misbitfield(new unsigned char[mislen]);
+  size_t blocks = bitfieldMan_->countBlock();
+  size_t misBlock = 0;
+  if(isEndGame()) {
+    bool r = bitfieldMan_->getAllMissingIndexes
+      (misbitfield, mislen, bitfield, length);
+    if(!r) {
+      return;
+    }
+    std::vector<size_t> indexes;
+    for(size_t i = 0; i < blocks; ++i) {
+      if(bitfield::test(misbitfield, blocks, i)) {
+        indexes.push_back(i);
+      }
+    }
+    std::random_shuffle(indexes.begin(), indexes.end());
+    for(std::vector<size_t>::const_iterator i = indexes.begin(),
+          eoi = indexes.end(); i != eoi && misBlock < minMissingBlocks; ++i) {
+      pieces.push_back(checkOutPiece(*i));
+      misBlock += pieces.back()->countMissingBlock();
     }
-  }
-}
-
-SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
-(const SharedHandle<Peer>& peer)
-{
-  if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
-    BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
-                             bitfieldMan->getTotalLength());
-    createFastIndexBitfield(tempBitfield, peer);
-    return getMissingPiece(tempBitfield);
   } else {
-    return SharedHandle<Piece>();
+    bool r = bitfieldMan_->getAllMissingUnusedIndexes
+      (misbitfield, mislen, bitfield, length);
+    if(!r) {
+      return;
+    }
+    while(misBlock < minMissingBlocks) {
+      size_t index;
+      if(pieceSelector_->select(index, misbitfield, blocks)) {
+        pieces.push_back(checkOutPiece(index));
+        bitfield::flipBit(misbitfield, blocks, index);
+        misBlock += pieces.back()->countMissingBlock();
+      } else {
+        break;
+      }
+    }
   }
 }
 
@@ -235,27 +212,119 @@ static void unsetExcludedIndexes(BitfieldMan& bitfield,
                 std::bind1st(std::mem_fun(&BitfieldMan::unsetBit), &bitfield));
 }
 
+void DefaultPieceStorage::createFastIndexBitfield
+(BitfieldMan& bitfield, const SharedHandle<Peer>& peer)
+{
+  for(std::vector<size_t>::const_iterator itr =
+        peer->getPeerAllowedIndexSet().begin(),
+        eoi = peer->getPeerAllowedIndexSet().end(); itr != eoi; ++itr) {
+    if(!bitfieldMan_->isBitSet(*itr) && peer->hasPiece(*itr)) {
+      bitfield.setBit(*itr);
+    }
+  }
+}
+
+void DefaultPieceStorage::getMissingPiece
+(std::vector<SharedHandle<Piece> >& pieces,
+ size_t minMissingBlocks,
+ const SharedHandle<Peer>& peer)
+{
+  getMissingPiece(pieces, minMissingBlocks,
+                  peer->getBitfield(), peer->getBitfieldLength());
+}
+
+
+void DefaultPieceStorage::getMissingPiece
+(std::vector<SharedHandle<Piece> >& pieces,
+ size_t minMissingBlocks,
+ const SharedHandle<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes)
+{
+  BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
+                           bitfieldMan_->getTotalLength());
+  tempBitfield.setBitfield(peer->getBitfield(), peer->getBitfieldLength());
+  unsetExcludedIndexes(tempBitfield, excludedIndexes);
+  getMissingPiece(pieces, minMissingBlocks,
+                  tempBitfield.getBitfield(), tempBitfield.getBitfieldLength());
+}
+
+void DefaultPieceStorage::getMissingFastPiece
+(std::vector<SharedHandle<Piece> >& pieces,
+ size_t minMissingBlocks,
+ const SharedHandle<Peer>& peer)
+{
+  if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
+    BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
+                             bitfieldMan_->getTotalLength());
+    createFastIndexBitfield(tempBitfield, peer);
+    getMissingPiece(pieces, minMissingBlocks,
+                    tempBitfield.getBitfield(),
+                    tempBitfield.getBitfieldLength());
+  }
+}
+
+void DefaultPieceStorage::getMissingFastPiece
+(std::vector<SharedHandle<Piece> >& pieces,
+ size_t minMissingBlocks,
+ const SharedHandle<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes)
+{
+  if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
+    BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
+                             bitfieldMan_->getTotalLength());
+    createFastIndexBitfield(tempBitfield, peer);
+    unsetExcludedIndexes(tempBitfield, excludedIndexes);
+    getMissingPiece(pieces, minMissingBlocks,
+                    tempBitfield.getBitfield(),
+                    tempBitfield.getBitfieldLength());
+  }
+}
+
+SharedHandle<Piece>
+DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
+{
+  std::vector<SharedHandle<Piece> > pieces;
+  getMissingPiece(pieces, 1, peer);
+  if(pieces.empty()) {
+    return SharedHandle<Piece>();
+  } else {
+    return pieces.front();
+  }
+}
+
 SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
 (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
 {
-  BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
-                           bitfieldMan->getTotalLength());
-  tempBitfield.setBitfield(peer->getBitfield(), peer->getBitfieldLength());
-  unsetExcludedIndexes(tempBitfield, excludedIndexes);
-  return getMissingPiece(tempBitfield);
+  std::vector<SharedHandle<Piece> > pieces;
+  getMissingPiece(pieces, 1, peer, excludedIndexes);
+  if(pieces.empty()) {
+    return SharedHandle<Piece>();
+  } else {
+    return pieces.front();
+  }
+}
+
+SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
+(const SharedHandle<Peer>& peer)
+{
+  std::vector<SharedHandle<Piece> > pieces;
+  getMissingFastPiece(pieces, 1, peer);
+  if(pieces.empty()) {
+    return SharedHandle<Piece>();
+  } else {
+    return pieces.front();
+  }
 }
 
 SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
 (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
 {
-  if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
-    BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
-                             bitfieldMan->getTotalLength());
-    createFastIndexBitfield(tempBitfield, peer);
-    unsetExcludedIndexes(tempBitfield, excludedIndexes);
-    return getMissingPiece(tempBitfield);
-  } else {
+  std::vector<SharedHandle<Piece> > pieces;
+  getMissingFastPiece(pieces, 1, peer, excludedIndexes);
+  if(pieces.empty()) {
     return SharedHandle<Piece>();
+  } else {
+    return pieces.front();
   }
 }
 
@@ -264,14 +333,15 @@ SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
 bool DefaultPieceStorage::hasMissingUnusedPiece()
 {
   size_t index;
-  return bitfieldMan->getFirstMissingUnusedIndex(index);
+  return bitfieldMan_->getFirstMissingUnusedIndex(index);
 }
 
 SharedHandle<Piece> DefaultPieceStorage::getSparseMissingUnusedPiece
-(const unsigned char* ignoreBitfield, size_t length)
+(size_t minSplitSize, const unsigned char* ignoreBitfield, size_t length)
 {
   size_t index;
-  if(bitfieldMan->getSparseMissingUnusedIndex(index, ignoreBitfield, length)) {
+  if(bitfieldMan_->getSparseMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length)) {
     return checkOutPiece(index);
   } else {
     return SharedHandle<Piece>();
@@ -293,9 +363,9 @@ void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
     return;
   }
   std::deque<SharedHandle<Piece> >::iterator i = 
-    std::lower_bound(usedPieces.begin(), usedPieces.end(), piece);
-  if(i != usedPieces.end() && (*i) == piece) {
-    usedPieces.erase(i);
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece);
+  if(i != usedPieces_.end() && (*i) == piece) {
+    usedPieces_.erase(i);
   }
 }
 
@@ -347,25 +417,35 @@ void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
   if(allDownloadFinished()) {
     return;
   }
-  bitfieldMan->setBit(piece->getIndex());
-  bitfieldMan->unsetUseBit(piece->getIndex());
+  bitfieldMan_->setBit(piece->getIndex());
+  bitfieldMan_->unsetUseBit(piece->getIndex());
   addPieceStats(piece->getIndex());
   if(downloadFinished()) {
-    downloadContext->resetDownloadStopTime();
+    downloadContext_->resetDownloadStopTime();
     if(isSelectiveDownloadingMode()) {
-      logger->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
+      logger_->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
       // following line was commented out in order to stop sending request
       // message after user-specified files were downloaded.
       //finishSelectiveDownloadingMode();
     } else {
-      logger->info(MSG_DOWNLOAD_COMPLETED);
+      logger_->info(MSG_DOWNLOAD_COMPLETED);
     }
+#ifdef ENABLE_BITTORRENT
+    if(downloadContext_->hasAttribute(bittorrent::BITTORRENT)) {
+      SharedHandle<TorrentAttribute> torrentAttrs =
+        bittorrent::getTorrentAttrs(downloadContext_);
+      if(!torrentAttrs->metadata.empty()) {
+        util::executeHookByOptName(downloadContext_->getOwnerRequestGroup(),
+                                   option_, PREF_ON_BT_DOWNLOAD_COMPLETE);
+      }
+    }
+#endif // ENABLE_BITTORRENT
   }
 }
 
 bool DefaultPieceStorage::isSelectiveDownloadingMode()
 {
-  return bitfieldMan->isFilterEnabled();
+  return bitfieldMan_->isFilterEnabled();
 }
 
 // not unittested
@@ -374,7 +454,7 @@ void DefaultPieceStorage::cancelPiece(const SharedHandle<Piece>& piece)
   if(piece.isNull()) {
     return;
   }
-  bitfieldMan->unsetUseBit(piece->getIndex());
+  bitfieldMan_->unsetUseBit(piece->getIndex());
   if(!isEndGame()) {
     if(piece->getCompletedLength() == 0) {
       deleteUsedPiece(piece);
@@ -384,28 +464,28 @@ void DefaultPieceStorage::cancelPiece(const SharedHandle<Piece>& piece)
 
 bool DefaultPieceStorage::hasPiece(size_t index)
 {
-  return bitfieldMan->isBitSet(index);
+  return bitfieldMan_->isBitSet(index);
 }
 
 bool DefaultPieceStorage::isPieceUsed(size_t index)
 {
-  return bitfieldMan->isUseBitSet(index);
+  return bitfieldMan_->isUseBitSet(index);
 }
 
 uint64_t DefaultPieceStorage::getTotalLength()
 {
-  return bitfieldMan->getTotalLength();
+  return bitfieldMan_->getTotalLength();
 }
 
 uint64_t DefaultPieceStorage::getFilteredTotalLength()
 {
-  return bitfieldMan->getFilteredTotalLength();
+  return bitfieldMan_->getFilteredTotalLength();
 }
 
 uint64_t DefaultPieceStorage::getCompletedLength()
 {
   uint64_t completedLength =
-    bitfieldMan->getCompletedLength()+getInFlightPieceCompletedLength();
+    bitfieldMan_->getCompletedLength()+getInFlightPieceCompletedLength();
   uint64_t totalLength = getTotalLength();
   if(completedLength > totalLength) {
     completedLength = totalLength;
@@ -415,19 +495,22 @@ uint64_t DefaultPieceStorage::getCompletedLength()
 
 uint64_t DefaultPieceStorage::getFilteredCompletedLength()
 {
-  return bitfieldMan->getFilteredCompletedLength()+getInFlightPieceCompletedLength();
+  return bitfieldMan_->getFilteredCompletedLength()+
+    getInFlightPieceCompletedLength();
 }
 
 size_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
 {
-  return std::accumulate(usedPieces.begin(), usedPieces.end(), 0, adopt2nd(std::plus<size_t>(), mem_fun_sh(&Piece::getCompletedLength)));
+  return std::accumulate(usedPieces_.begin(), usedPieces_.end(),
+                         0, adopt2nd(std::plus<size_t>(),
+                                     mem_fun_sh(&Piece::getCompletedLength)));
 }
 
 // not unittested
 void DefaultPieceStorage::setupFileFilter()
 {
   const std::vector<SharedHandle<FileEntry> >& fileEntries =
-    downloadContext->getFileEntries();
+    downloadContext_->getFileEntries();
   bool allSelected = true;
   for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
         fileEntries.begin(), eoi = fileEntries.end();
@@ -443,122 +526,124 @@ void DefaultPieceStorage::setupFileFilter()
   for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
         fileEntries.begin(), eoi = fileEntries.end(); i != eoi; ++i) {
     if((*i)->isRequested()) {
-      bitfieldMan->addFilter((*i)->getOffset(), (*i)->getLength());
+      bitfieldMan_->addFilter((*i)->getOffset(), (*i)->getLength());
     }
   }
-  bitfieldMan->enableFilter();
+  bitfieldMan_->enableFilter();
 }
 
 // not unittested
 void DefaultPieceStorage::clearFileFilter()
 {
-  bitfieldMan->clearFilter();
+  bitfieldMan_->clearFilter();
 }
 
 // not unittested
 bool DefaultPieceStorage::downloadFinished()
 {
-  // TODO iterate all requested FileEntry and Call bitfieldMan->isBitSetOffsetRange()
-  return bitfieldMan->isFilteredAllBitSet();
+  // TODO iterate all requested FileEntry and Call
+  // bitfieldMan->isBitSetOffsetRange()
+  return bitfieldMan_->isFilteredAllBitSet();
 }
 
 // not unittested
 bool DefaultPieceStorage::allDownloadFinished()
 {
-  return bitfieldMan->isAllBitSet();
+  return bitfieldMan_->isAllBitSet();
 }
 
 // not unittested
 void DefaultPieceStorage::initStorage()
 {
-  if(downloadContext->getFileEntries().size() == 1) {
-    if(logger->debug()) {
-      logger->debug("Instantiating DirectDiskAdaptor");
+  if(downloadContext_->getFileEntries().size() == 1) {
+    if(logger_->debug()) {
+      logger_->debug("Instantiating DirectDiskAdaptor");
     }
     DirectDiskAdaptorHandle directDiskAdaptor(new DirectDiskAdaptor());
-    directDiskAdaptor->setTotalLength(downloadContext->getTotalLength());
-    directDiskAdaptor->setFileEntries(downloadContext->getFileEntries().begin(),
-                                      downloadContext->getFileEntries().end());
+    directDiskAdaptor->setTotalLength(downloadContext_->getTotalLength());
+    directDiskAdaptor->setFileEntries
+      (downloadContext_->getFileEntries().begin(),
+       downloadContext_->getFileEntries().end());
 
     DiskWriterHandle writer =
-      _diskWriterFactory->newDiskWriter(directDiskAdaptor->getFilePath());
-    if(option->getAsBool(PREF_ENABLE_DIRECT_IO)) {
+      diskWriterFactory_->newDiskWriter(directDiskAdaptor->getFilePath());
+    if(option_->getAsBool(PREF_ENABLE_DIRECT_IO)) {
       writer->allowDirectIO();
     }
 
     directDiskAdaptor->setDiskWriter(writer);
-    this->diskAdaptor = directDiskAdaptor;
+    diskAdaptor_ = directDiskAdaptor;
   } else {
-    if(logger->debug()) {
-      logger->debug("Instantiating MultiDiskAdaptor");
+    if(logger_->debug()) {
+      logger_->debug("Instantiating MultiDiskAdaptor");
     }
     MultiDiskAdaptorHandle multiDiskAdaptor(new MultiDiskAdaptor());
-    multiDiskAdaptor->setFileEntries(downloadContext->getFileEntries().begin(),
-                                     downloadContext->getFileEntries().end());
-    if(option->getAsBool(PREF_ENABLE_DIRECT_IO)) {
+    multiDiskAdaptor->setFileEntries(downloadContext_->getFileEntries().begin(),
+                                     downloadContext_->getFileEntries().end());
+    if(option_->getAsBool(PREF_ENABLE_DIRECT_IO)) {
       multiDiskAdaptor->allowDirectIO();
     }
-    multiDiskAdaptor->setPieceLength(downloadContext->getPieceLength());
-    multiDiskAdaptor->setMaxOpenFiles(option->getAsInt(PREF_BT_MAX_OPEN_FILES));
-    this->diskAdaptor = multiDiskAdaptor;
+    multiDiskAdaptor->setPieceLength(downloadContext_->getPieceLength());
+    multiDiskAdaptor->setMaxOpenFiles
+      (option_->getAsInt(PREF_BT_MAX_OPEN_FILES));
+    diskAdaptor_ = multiDiskAdaptor;
   }
-#ifdef HAVE_POSIX_FALLOCATE
-  if(option->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
-    diskAdaptor->enableFallocate();
+  if(option_->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
+    diskAdaptor_->enableFallocate();
   }
-#endif // HAVE_POSIX_FALLOCATE
 }
 
 void DefaultPieceStorage::setBitfield(const unsigned char* bitfield,
                                       size_t bitfieldLength)
 {
-  bitfieldMan->setBitfield(bitfield, bitfieldLength);
+  bitfieldMan_->setBitfield(bitfield, bitfieldLength);
   addPieceStats(bitfield, bitfieldLength);
 }
 
 size_t DefaultPieceStorage::getBitfieldLength()
 {
-  return bitfieldMan->getBitfieldLength();
+  return bitfieldMan_->getBitfieldLength();
 }
 
 const unsigned char* DefaultPieceStorage::getBitfield()
 {
-  return bitfieldMan->getBitfield();
+  return bitfieldMan_->getBitfield();
 }
 
 DiskAdaptorHandle DefaultPieceStorage::getDiskAdaptor() {
-  return diskAdaptor;
+  return diskAdaptor_;
 }
 
 size_t DefaultPieceStorage::getPieceLength(size_t index)
 {
-  return bitfieldMan->getBlockLength(index);
+  return bitfieldMan_->getBlockLength(index);
 }
 
-void DefaultPieceStorage::advertisePiece(int32_t cuid, size_t index)
+void DefaultPieceStorage::advertisePiece(cuid_t cuid, size_t index)
 {
-  HaveEntry entry(cuid, index);
-  haves.push_front(entry);
+  HaveEntry entry(cuid, index, global::wallclock);
+  haves_.push_front(entry);
 }
 
 void
 DefaultPieceStorage::getAdvertisedPieceIndexes(std::vector<size_t>& indexes,
-                                               int32_t myCuid,
-                                               const Time& lastCheckTime)
+                                               cuid_t myCuid,
+                                               const Timer& lastCheckTime)
 {
-  for(std::deque<HaveEntry>::const_iterator itr = haves.begin(),
-        eoi = haves.end(); itr != eoi; ++itr) {
+  for(std::deque<HaveEntry>::const_iterator itr = haves_.begin(),
+        eoi = haves_.end(); itr != eoi; ++itr) {
     const HaveEntry& have = *itr;
     if(have.getCuid() == myCuid) {
       continue;
     }
-    if(lastCheckTime.isNewer(have.getRegisteredTime())) {
+    if(lastCheckTime > have.getRegisteredTime()) {
       break;
     }
     indexes.push_back(have.getIndex());
   }
 }
 
+namespace {
 class FindElapsedHave
 {
 private:
@@ -574,37 +659,42 @@ public:
     }
   }
 };
-  
+}
+
 void DefaultPieceStorage::removeAdvertisedPiece(time_t elapsed)
 {
   std::deque<HaveEntry>::iterator itr =
-    std::find_if(haves.begin(), haves.end(), FindElapsedHave(elapsed));
-  if(itr != haves.end()) {
-    if(logger->debug()) {
-      logger->debug(MSG_REMOVED_HAVE_ENTRY, haves.end()-itr);
+    std::find_if(haves_.begin(), haves_.end(), FindElapsedHave(elapsed));
+  if(itr != haves_.end()) {
+    if(logger_->debug()) {
+      logger_->debug(MSG_REMOVED_HAVE_ENTRY, haves_.end()-itr);
     }
-    haves.erase(itr, haves.end());
+    haves_.erase(itr, haves_.end());
   }
 }
 
 void DefaultPieceStorage::markAllPiecesDone()
 {
-  bitfieldMan->setAllBit();
+  bitfieldMan_->setAllBit();
 }
 
 void DefaultPieceStorage::markPiecesDone(uint64_t length)
 {
-  if(length == bitfieldMan->getTotalLength()) {
-    bitfieldMan->setAllBit();
+  if(length == bitfieldMan_->getTotalLength()) {
+    bitfieldMan_->setAllBit();
+  } else if(length == 0) {
+    // TODO this would go to markAllPiecesUndone()
+    bitfieldMan_->clearAllBit();
+    usedPieces_.clear();
   } else {
-    size_t numPiece = length/bitfieldMan->getBlockLength();
+    size_t numPiece = length/bitfieldMan_->getBlockLength();
     if(numPiece > 0) {
-      bitfieldMan->setBitRange(0, numPiece-1);
+      bitfieldMan_->setBitRange(0, numPiece-1);
     }
-    size_t r = (length%bitfieldMan->getBlockLength())/Piece::BLOCK_LENGTH;
+    size_t r = (length%bitfieldMan_->getBlockLength())/Piece::BLOCK_LENGTH;
     if(r > 0) {
       SharedHandle<Piece> p
-        (new Piece(numPiece, bitfieldMan->getBlockLength(numPiece)));
+        (new Piece(numPiece, bitfieldMan_->getBlockLength(numPiece)));
       
       for(size_t i = 0; i < r; ++i) {
         p->completeBlock(i);
@@ -612,7 +702,7 @@ void DefaultPieceStorage::markPiecesDone(uint64_t length)
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-      p->setHashAlgo(downloadContext->getPieceHashAlgo());
+      p->setHashAlgo(downloadContext_->getPieceHashAlgo());
 
 #endif // ENABLE_MESSAGE_DIGEST
 
@@ -623,55 +713,66 @@ void DefaultPieceStorage::markPiecesDone(uint64_t length)
 
 void DefaultPieceStorage::markPieceMissing(size_t index)
 {
-  bitfieldMan->unsetBit(index);
+  bitfieldMan_->unsetBit(index);
 }
 
 void DefaultPieceStorage::addInFlightPiece
 (const std::vector<SharedHandle<Piece> >& pieces)
 {
-  usedPieces.insert(usedPieces.end(), pieces.begin(), pieces.end());
-  std::sort(usedPieces.begin(), usedPieces.end());
+  usedPieces_.insert(usedPieces_.end(), pieces.begin(), pieces.end());
+  std::sort(usedPieces_.begin(), usedPieces_.end());
 }
 
 size_t DefaultPieceStorage::countInFlightPiece()
 {
-  return usedPieces.size();
+  return usedPieces_.size();
 }
 
 void DefaultPieceStorage::getInFlightPieces
 (std::vector<SharedHandle<Piece> >& pieces)
 {
-  pieces.insert(pieces.end(), usedPieces.begin(), usedPieces.end());
+  pieces.insert(pieces.end(), usedPieces_.begin(), usedPieces_.end());
 }
 
-void DefaultPieceStorage::setDiskWriterFactory(const DiskWriterFactoryHandle& diskWriterFactory)
+void DefaultPieceStorage::setDiskWriterFactory
+(const DiskWriterFactoryHandle& diskWriterFactory)
 {
-  _diskWriterFactory = diskWriterFactory;
+  diskWriterFactory_ = diskWriterFactory;
 }
 
 void DefaultPieceStorage::addPieceStats(const unsigned char* bitfield,
                                         size_t bitfieldLength)
 {
-  _pieceStatMan->addPieceStats(bitfield, bitfieldLength);
+  pieceStatMan_->addPieceStats(bitfield, bitfieldLength);
 }
 
 void DefaultPieceStorage::subtractPieceStats(const unsigned char* bitfield,
                                              size_t bitfieldLength)
 {
-  _pieceStatMan->subtractPieceStats(bitfield, bitfieldLength);
+  pieceStatMan_->subtractPieceStats(bitfield, bitfieldLength);
 }
 
 void DefaultPieceStorage::updatePieceStats(const unsigned char* newBitfield,
                                            size_t newBitfieldLength,
                                            const unsigned char* oldBitfield)
 {
-  _pieceStatMan->updatePieceStats(newBitfield, newBitfieldLength,
+  pieceStatMan_->updatePieceStats(newBitfield, newBitfieldLength,
                                   oldBitfield);
 }
 
 void DefaultPieceStorage::addPieceStats(size_t index)
 {
-  _pieceStatMan->addPieceStats(index);
+  pieceStatMan_->addPieceStats(index);
+}
+
+size_t DefaultPieceStorage::getNextUsedIndex(size_t index)
+{
+  for(size_t i = index+1; i < bitfieldMan_->countBlock(); ++i) {
+    if(bitfieldMan_->isUseBitSet(i) || bitfieldMan_->isBitSet(i)) {
+      return i;
+    }
+  }
+  return bitfieldMan_->countBlock();
 }
 
 } // namespace aria2
