@@ -11,7 +11,7 @@
 #include "DHTTokenTracker.h"
 #include "DHTPeerAnnounceStorage.h"
 #include "DHTRoutingTable.h"
-#include "bencode.h"
+#include "bencode2.h"
 
 namespace aria2 {
 
@@ -31,33 +31,22 @@ public:
 
   class MockDHTMessageFactory2:public MockDHTMessageFactory {
   public:
-    virtual SharedHandle<DHTMessage>
-    createGetPeersReplyMessage(const SharedHandle<DHTNode>& remoteNode,
-                               const std::vector<SharedHandle<Peer> >& peers,
-                               const std::string& token,
-                               const std::string& transactionID)
-    {
-      SharedHandle<MockDHTMessage> m
-        (new MockDHTMessage(_localNode, remoteNode, "get_peers", transactionID));
-      m->_peers = peers;
-      m->_token = token;
-      return m;
-    }
-
-    virtual SharedHandle<DHTMessage>
+    virtual SharedHandle<DHTResponseMessage>
     createGetPeersReplyMessage
     (const SharedHandle<DHTNode>& remoteNode,
      const std::vector<SharedHandle<DHTNode> >& closestKNodes,
+     const std::vector<SharedHandle<Peer> >& peers,
      const std::string& token,
      const std::string& transactionID)
     {
-      SharedHandle<MockDHTMessage> m
-        (new MockDHTMessage(_localNode, remoteNode, "get_peers", transactionID));
-      m->_nodes = closestKNodes;
-      m->_token = token;
+      SharedHandle<MockDHTResponseMessage> m
+        (new MockDHTResponseMessage
+         (localNode_, remoteNode, "get_peers", transactionID));
+      m->nodes_ = closestKNodes;
+      m->peers_ = peers;
+      m->token_ = token;
       return m;
     }
-
   };
 };
 
@@ -81,17 +70,17 @@ void DHTGetPeersMessageTest::testGetBencodedMessage()
 
   std::string msgbody = msg.getBencodedMessage();
 
-  BDE dict = BDE::dict();
-  dict["t"] = transactionID;
-  dict["v"] = BDE("A200");
-  dict["y"] = BDE("q");
-  dict["q"] = BDE("get_peers");
-  BDE aDict = BDE::dict();
-  aDict["id"] = BDE(localNode->getID(), DHT_ID_LENGTH);
-  aDict["info_hash"] = BDE(infoHash, DHT_ID_LENGTH);
-  dict["a"] = aDict;
+  Dict dict;
+  dict.put("t", transactionID);
+  dict.put("v", "A200");
+  dict.put("y", "q");
+  dict.put("q", "get_peers");
+  SharedHandle<Dict> aDict = Dict::g();
+  aDict->put("id", String::g(localNode->getID(), DHT_ID_LENGTH));
+  aDict->put("info_hash", String::g(infoHash, DHT_ID_LENGTH));
+  dict.put("a", aDict);
 
-  CPPUNIT_ASSERT_EQUAL(util::percentEncode(bencode::encode(dict)),
+  CPPUNIT_ASSERT_EQUAL(util::percentEncode(bencode2::encode(&dict)),
                        util::percentEncode(msgbody));
 }
 
@@ -113,14 +102,13 @@ void DHTGetPeersMessageTest::testDoReceivedAction()
   MockDHTMessageDispatcher dispatcher;
   MockDHTMessageFactory2 factory;
   factory.setLocalNode(localNode);
+  DHTRoutingTable routingTable(localNode);
 
   DHTGetPeersMessage msg(localNode, remoteNode, infoHash, transactionID);
-  msg.setTokenTracker(WeakHandle<DHTTokenTracker>
-                      (&tokenTracker));
-  msg.setMessageDispatcher(WeakHandle<DHTMessageDispatcher>
-                           (&dispatcher));
-  msg.setMessageFactory(WeakHandle<DHTMessageFactory>
-                        (&factory));
+  msg.setRoutingTable(WeakHandle<DHTRoutingTable>(&routingTable));
+  msg.setTokenTracker(WeakHandle<DHTTokenTracker>(&tokenTracker));
+  msg.setMessageDispatcher(WeakHandle<DHTMessageDispatcher>(&dispatcher));
+  msg.setMessageFactory(WeakHandle<DHTMessageFactory>(&factory));
   {
     // localhost has peer contact information for that infohash.
     DHTPeerAnnounceStorage peerAnnounceStorage;
@@ -132,28 +120,29 @@ void DHTGetPeersMessageTest::testDoReceivedAction()
   
     msg.doReceivedAction();
 
-    CPPUNIT_ASSERT_EQUAL((size_t)1, dispatcher._messageQueue.size());
-    SharedHandle<MockDHTMessage> m
-      (dynamic_pointer_cast<MockDHTMessage>(dispatcher._messageQueue[0]._message));
+    CPPUNIT_ASSERT_EQUAL((size_t)1, dispatcher.messageQueue_.size());
+    SharedHandle<MockDHTResponseMessage> m
+      (dynamic_pointer_cast<MockDHTResponseMessage>
+       (dispatcher.messageQueue_[0].message_));
     CPPUNIT_ASSERT(localNode == m->getLocalNode());
     CPPUNIT_ASSERT(remoteNode == m->getRemoteNode());
     CPPUNIT_ASSERT_EQUAL(std::string("get_peers"), m->getMessageType());
     CPPUNIT_ASSERT_EQUAL(msg.getTransactionID(), m->getTransactionID());
-    CPPUNIT_ASSERT_EQUAL(tokenTracker.generateToken(infoHash, remoteNode->getIPAddress(), remoteNode->getPort()), m->_token);
-    CPPUNIT_ASSERT_EQUAL((size_t)0, m->_nodes.size());
-    CPPUNIT_ASSERT_EQUAL((size_t)2, m->_peers.size());
+    CPPUNIT_ASSERT_EQUAL(tokenTracker.generateToken(infoHash, remoteNode->getIPAddress(), remoteNode->getPort()), m->token_);
+    CPPUNIT_ASSERT_EQUAL((size_t)0, m->nodes_.size());
+    CPPUNIT_ASSERT_EQUAL((size_t)2, m->peers_.size());
     {
-      SharedHandle<Peer> peer = m->_peers[0];
-      CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.100"), peer->ipaddr);
-      CPPUNIT_ASSERT_EQUAL((uint16_t)6888, peer->port);
+      SharedHandle<Peer> peer = m->peers_[0];
+      CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.100"), peer->getIPAddress());
+      CPPUNIT_ASSERT_EQUAL((uint16_t)6888, peer->getPort());
     }
     {
-      SharedHandle<Peer> peer = m->_peers[1];
-      CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.101"), peer->ipaddr);
-      CPPUNIT_ASSERT_EQUAL((uint16_t)6889, peer->port);
+      SharedHandle<Peer> peer = m->peers_[1];
+      CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.101"), peer->getIPAddress());
+      CPPUNIT_ASSERT_EQUAL((uint16_t)6889, peer->getPort());
     }
   }
-  dispatcher._messageQueue.clear();
+  dispatcher.messageQueue_.clear();
   {
     // localhost doesn't have peer contact information for that infohash.
     DHTPeerAnnounceStorage peerAnnounceStorage;
@@ -168,17 +157,18 @@ void DHTGetPeersMessageTest::testDoReceivedAction()
 
     msg.doReceivedAction();
 
-    CPPUNIT_ASSERT_EQUAL((size_t)1, dispatcher._messageQueue.size());
-    SharedHandle<MockDHTMessage> m
-      (dynamic_pointer_cast<MockDHTMessage>(dispatcher._messageQueue[0]._message));
+    CPPUNIT_ASSERT_EQUAL((size_t)1, dispatcher.messageQueue_.size());
+    SharedHandle<MockDHTResponseMessage> m
+      (dynamic_pointer_cast<MockDHTResponseMessage>
+       (dispatcher.messageQueue_[0].message_));
     CPPUNIT_ASSERT(localNode == m->getLocalNode());
     CPPUNIT_ASSERT(remoteNode == m->getRemoteNode());
     CPPUNIT_ASSERT_EQUAL(std::string("get_peers"), m->getMessageType());
     CPPUNIT_ASSERT_EQUAL(msg.getTransactionID(), m->getTransactionID());
-    CPPUNIT_ASSERT_EQUAL(tokenTracker.generateToken(infoHash, remoteNode->getIPAddress(), remoteNode->getPort()), m->_token);
-    CPPUNIT_ASSERT_EQUAL((size_t)1, m->_nodes.size());
-    CPPUNIT_ASSERT(returnNode1 == m->_nodes[0]);
-    CPPUNIT_ASSERT_EQUAL((size_t)0, m->_peers.size());
+    CPPUNIT_ASSERT_EQUAL(tokenTracker.generateToken(infoHash, remoteNode->getIPAddress(), remoteNode->getPort()), m->token_);
+    CPPUNIT_ASSERT_EQUAL((size_t)1, m->nodes_.size());
+    CPPUNIT_ASSERT(returnNode1 == m->nodes_[0]);
+    CPPUNIT_ASSERT_EQUAL((size_t)0, m->peers_.size());
   }
 }
 
